@@ -46,6 +46,32 @@ func TestLoadExpr(t *testing.T) {
 	}
 }
 
+func TestSpriteOf(t *testing.T) {
+	proj := &gmxProject{}
+	if proj.spriteOf("a") != nil {
+		t.Fatal("spriteOf: not nil?")
+	}
+}
+
+func TestGetGameClass(t *testing.T) {
+	proj := &gmxProject{
+		gameIsPtr:  true,
+		hasMain_:   false,
+		gameClass_: "",
+		gt: &modfile.Project{
+			Class:    "*App",
+			PkgPaths: []string{"foo/bar"},
+		},
+	}
+	ctx := &pkgCtx{
+		projs: map[string]*gmxProject{".gmx": proj, ".spx": proj},
+		nproj: 2,
+	}
+	if v := proj.getGameClass(ctx); v != "BarApp" {
+		t.Fatal("getGameClass:", v)
+	}
+}
+
 func TestSimplifyPkgPath(t *testing.T) {
 	if simplifyPkgPath("c/lua") != "github.com/goplus/llgo/c/lua" {
 		t.Fatal("simplifyPkgPath: c/lua")
@@ -492,20 +518,9 @@ func TestGmxProject(t *testing.T) {
 		t.Fatal("TestGmxProject failed: hasScheds?")
 	}
 
-	/* _, err := NewPackage("", &ast.Package{Files: map[string]*ast.File{
-		"main.t2gmx": {
-			IsProj: true,
-		},
-	}}, &Config{
-		LookupClass: lookupClassErr,
-	})
-	if e := err.Error(); e != `github.com/goplus/gop/cl/internal/libc.Game not found` {
-		t.Fatal("newGmx:", e)
-	} */
-
 	func() {
 		defer func() {
-			if e := recover(); e != "TODO: class not found" {
+			if e := recover(); e != "class not found: .abcx" {
 				t.Fatal("TestGmxProject failed:", e)
 			}
 		}()
@@ -515,7 +530,7 @@ func TestGmxProject(t *testing.T) {
 	}()
 	func() {
 		defer func() {
-			if e := recover(); e != "multiple project files found: Game Game\n" {
+			if e := recover(); e != "multiple project files found: main, main" {
 				t.Fatal("TestGmxProject failed:", e)
 			}
 		}()
@@ -604,7 +619,6 @@ func TestClassFileEnd(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	f.Decls = append(f.Decls, astFnClassfname(&gmxClass{clsfile: "get"}))
 	if f.End() != f.ShadowEntry.End() {
 		t.Fatal("class file end not shadow entry")
 	}
