@@ -332,6 +332,7 @@ func doInitMethods(ld *typeLoader) {
 
 type pkgCtx struct {
 	*nodeInterp
+	nproj    int                    // number of non-test projects
 	projs    map[string]*gmxProject // .gmx => project
 	classes  map[*ast.File]*gmxClass
 	overpos  map[string]token.Pos // overload => pos
@@ -368,6 +369,7 @@ type blockCtx struct {
 	pystr_     gogen.Ref
 	relBaseDir string
 
+	classDecl *ast.GenDecl   // available when isClass
 	classRecv *ast.FieldList // available when isClass
 	baseClass types.Object   // available when isClass
 
@@ -616,7 +618,7 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gogen.Packag
 	if genMain { // make classfile main func if need
 		if proj != nil {
 			if !multi { // only one project file
-				mainClass = proj.gameClass
+				mainClass = proj.getGameClass(ctx)
 			}
 		} else if ctx.goxMain == 1 {
 			mainClass = ctx.goxMainClass // main func in normal gox file
@@ -722,14 +724,11 @@ func genGoFile(file string, goxTestFile bool) string {
 func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, conf *Config) {
 	var proj *gmxProject
 	var c *gmxClass
-	var classType string
+	var classType, gameClass string
 	var testType string
 	var baseTypeName string
 	var baseType types.Type
-	var spxProj string
-	var spxClass bool
-	var spxClassfname bool
-	var spxClassclone bool
+	var sp *spxObj
 	var goxTestFile bool
 	var parent = ctx.pkgCtx
 	if f.IsClass {
@@ -744,17 +743,18 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 			}
 		} else {
 			c = parent.classes[f]
-			classType = c.tname
 			proj, ctx.proj = c.proj, c.proj
+			classType, gameClass = c.getName(parent), proj.getGameClass(parent)
 			ctx.autoimps = proj.autoimps
 			goxTestFile = proj.isTest
 			if goxTestFile { // test classfile
-				testType = c.tname
+				testType = classType
 				if !f.IsProj {
 					classType = casePrefix + testNameSuffix(testType)
 				}
 			}
 			if f.IsProj {
+				classType = gameClass
 				o := proj.game
 				ctx.baseClass = o
 				baseTypeName, baseType = o.Name(), o.Type()
@@ -762,13 +762,10 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 					baseType = types.NewPointer(baseType)
 				}
 			} else {
-				sp := proj.sprite[c.ext]
+				sp = proj.spriteOf(c.ext)
 				o := sp.obj
 				ctx.baseClass = o
-				baseTypeName, baseType, spxProj = o.Name(), o.Type(), sp.proj
-				spxClassfname = (proj.spfeats & spriteClassfname) != 0
-				spxClassclone = (proj.spfeats & spriteClassclone) != 0
-				spxClass = true
+				baseTypeName, baseType = o.Name(), o.Type()
 			}
 		}
 	}
@@ -785,7 +782,7 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 		}
 		syms := parent.syms
 		pos := f.Pos()
-		specs := getFields(f)
+		ctx.classDecl = f.ClassFieldsDecl()
 		ld := getTypeLoader(parent, syms, pos, classType)
 		ld.typ = func() {
 			if debugLoad {
@@ -811,12 +808,11 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 					tags = append(tags, "")
 					chk.chkRedecl(ctx, baseTypeName, pos)
 				}
-				if spxClass {
-					if gameClass := proj.gameClass; gameClass != "" {
-						if spxProj == "" { // if spxProj is empty, use gameClass
-							spxProj = gameClass
-						}
-						typ := toType(ctx, &ast.StarExpr{X: &ast.Ident{Name: spxProj}})
+				if sp != nil && !goxTestFile {
+					if gameClass != "" {
+						typ := toType(ctx, &ast.Ident{Name: gameClass})
+						getUnderlying(ctx, typ) // ensure type is loaded
+						typ = types.NewPointer(typ)
 						name := getTypeName(typ)
 						if !chk.chkRedecl(ctx, name, pos) {
 							fld := types.NewField(pos, pkg, name, typ, true)
@@ -826,32 +822,34 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 					}
 				}
 				rec := ctx.recorder()
-				for _, v := range specs {
-					spec := v.(*ast.ValueSpec)
-					typ := toType(ctx, spec.Type)
-					tag := toFieldTag(spec.Tag)
-					if len(spec.Names) == 0 {
-						name := parseTypeEmbedName(spec.Type)
-						if chk.chkRedecl(ctx, name.Name, spec.Type.Pos()) {
-							continue
-						}
-						fld := types.NewField(spec.Type.Pos(), pkg, name.Name, typ, true)
-						if rec != nil {
-							rec.Def(name, fld)
-						}
-						flds = append(flds, fld)
-						tags = append(tags, tag)
-					} else {
-						for _, name := range spec.Names {
-							if chk.chkRedecl(ctx, name.Name, name.Pos()) {
+				if classDecl := ctx.classDecl; classDecl != nil {
+					for _, v := range classDecl.Specs {
+						spec := v.(*ast.ValueSpec)
+						typ := toType(ctx, spec.Type)
+						tag := toFieldTag(spec.Tag)
+						if len(spec.Names) == 0 {
+							name := parseTypeEmbedName(spec.Type)
+							if chk.chkRedecl(ctx, name.Name, spec.Type.Pos()) {
 								continue
 							}
-							fld := types.NewField(name.Pos(), pkg, name.Name, typ, false)
+							fld := types.NewField(spec.Type.Pos(), pkg, name.Name, typ, true)
 							if rec != nil {
 								rec.Def(name, fld)
 							}
 							flds = append(flds, fld)
 							tags = append(tags, tag)
+						} else {
+							for _, name := range spec.Names {
+								if chk.chkRedecl(ctx, name.Name, name.Pos()) {
+									continue
+								}
+								fld := types.NewField(name.Pos(), pkg, name.Name, typ, false)
+								if rec != nil {
+									rec.Def(name, fld)
+								}
+								flds = append(flds, fld)
+								tags = append(tags, tag)
+							}
 						}
 					}
 				}
@@ -871,14 +869,6 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 				X: &ast.Ident{Name: classType},
 			},
 		}}}
-		// func Classfname() string
-		if spxClassfname {
-			f.Decls = append(f.Decls, astFnClassfname(c))
-		}
-		// func Classclone() any
-		if spxClassclone {
-			f.Decls = append(f.Decls, astFnClassclone())
-		}
 	}
 
 	if d := f.ShadowEntry; d != nil {
@@ -888,6 +878,26 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 	}
 
 	preloadFile(p, ctx, f, goFile, !conf.Outline)
+	if sp != nil && sp.feats != 0 {
+		spfeats := sp.feats
+		ld := getTypeLoader(parent, parent.syms, token.NoPos, classType)
+		if (spfeats & spriteClassfname) != 0 { // Classfname() string
+			ld.methods = append(ld.methods, func() {
+				old, _ := p.SetCurFile(goFile, true)
+				defer p.RestoreCurFile(old)
+				doInitType(ld)
+				genClassfname(ctx, c)
+			})
+		}
+		if (spfeats & spriteClassclone) != 0 { // Classclone() clonetype
+			ld.methods = append(ld.methods, func() {
+				old, _ := p.SetCurFile(goFile, true)
+				defer p.RestoreCurFile(old)
+				doInitType(ld)
+				genClassclone(ctx, sp.clone)
+			})
+		}
+	}
 	if goxTestFile {
 		parent.inits = append(parent.inits, func() {
 			old, _ := p.SetCurFile(testingGoFile, true)
@@ -913,13 +923,10 @@ retry:
 
 func preloadFile(p *gogen.Package, ctx *blockCtx, f *ast.File, goFile string, genFnBody bool) {
 	parent := ctx.pkgCtx
+	classDecl := ctx.classDecl
 	syms := parent.syms
 	old, _ := p.SetCurFile(goFile, true)
 	defer p.RestoreCurFile(old)
-	var skipClassFields bool
-	if f.IsClass {
-		skipClassFields = true
-	}
 
 	preloadFuncDecl := func(d *ast.FuncDecl) {
 		if ctx.classRecv != nil { // in class file (.spx/.gmx)
@@ -1031,7 +1038,12 @@ func preloadFile(p *gogen.Package, ctx *blockCtx, f *ast.File, goFile string, ge
 								if debugLoad {
 									log.Println("==> Load > AliasType", name)
 								}
-								defs.AliasType(name, toType(ctx, t.Type), tName)
+								typ := defs.AliasType(name, toType(ctx, t.Type), tName)
+								if rec := ctx.recorder(); rec != nil {
+									if obj, ok := typ.(interface{ Obj() *types.TypeName }); ok {
+										rec.Def(tName, obj.Obj())
+									}
+								}
 								return
 							}
 							if debugLoad {
@@ -1081,8 +1093,7 @@ func preloadFile(p *gogen.Package, ctx *blockCtx, f *ast.File, goFile string, ge
 			case token.CONST:
 				preloadConst(d)
 			case token.VAR:
-				if skipClassFields {
-					skipClassFields = false
+				if d == classDecl { // skip class fields
 					continue
 				}
 				for _, spec := range d.Specs {
