@@ -32,6 +32,7 @@ import (
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/printer"
 	"github.com/goplus/gop/token"
+	tpl "github.com/goplus/gop/tpl/ast"
 )
 
 /*-----------------------------------------------------------------------------
@@ -66,6 +67,7 @@ const (
 	clCommandWithoutArgs // this expr is a command without args (eg. ls)
 	clCommandIdent       // this expr is a command and an ident (eg. mkdir "abc")
 	clIdentInStringLitEx // this expr is an ident in a string extended literal (eg. ${PATH})
+	clInCallExpr
 )
 
 const (
@@ -286,6 +288,9 @@ func identOrSelectorFlags(inFlags []int) (flags int, cmdNoArgs bool) {
 		return clIdentCanAutoCall, false
 	}
 	flags = inFlags[0]
+	if flags&clInCallExpr != 0 {
+		return
+	}
 	if cmdNoArgs = (flags & clCommandWithoutArgs) != 0; cmdNoArgs {
 		flags &^= clCommandWithoutArgs
 	} else {
@@ -356,9 +361,9 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 	case *ast.RangeExpr:
 		compileRangeExpr(ctx, v)
 	case *ast.IndexExpr:
-		compileIndexExpr(ctx, v, twoValue(inFlags))
+		compileIndexExpr(ctx, v, inFlags...)
 	case *ast.IndexListExpr:
-		compileIndexListExpr(ctx, v, twoValue(inFlags))
+		compileIndexListExpr(ctx, v, inFlags...)
 	case *ast.SliceExpr:
 		compileSliceExpr(ctx, v)
 	case *ast.StarExpr:
@@ -441,19 +446,19 @@ func compileTypeAssertExpr(ctx *blockCtx, v *ast.TypeAssertExpr, twoValue bool)
 	ctx.cb.TypeAssert(typ, twoValue, v)
 }
 
-func compileIndexExpr(ctx *blockCtx, v *ast.IndexExpr, twoValue bool) { // x[i]
-	compileExpr(ctx, v.X)
+func compileIndexExpr(ctx *blockCtx, v *ast.IndexExpr, inFlags ...int) { // x[i]
+	compileExpr(ctx, v.X, inFlags...)
 	compileExpr(ctx, v.Index)
-	ctx.cb.Index(1, twoValue, v)
+	ctx.cb.Index(1, twoValue(inFlags), v)
 }
 
-func compileIndexListExpr(ctx *blockCtx, v *ast.IndexListExpr, twoValue bool) { // fn[t1,t2]
-	compileExpr(ctx, v.X)
+func compileIndexListExpr(ctx *blockCtx, v *ast.IndexListExpr, inFlags ...int) { // fn[t1,t2]
+	compileExpr(ctx, v.X, inFlags...)
 	n := len(v.Indices)
 	for i := 0; i < n; i++ {
 		compileExpr(ctx, v.Indices[i])
 	}
-	ctx.cb.Index(n, twoValue, v)
+	ctx.cb.Index(n, twoValue(inFlags), v)
 }
 
 func compileSliceExpr(ctx *blockCtx, v *ast.SliceExpr) { // x[i:j:k]
@@ -681,7 +686,7 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 		}
 		compileErrWrapExpr(ctx, fn, 0)
 	default:
-		compileExpr(ctx, fn)
+		compileExpr(ctx, fn, clInCallExpr)
 	}
 	var err error
 	var stk = ctx.cb.InternalStack()
@@ -873,14 +878,14 @@ retry:
 
 func compileLambda(ctx *blockCtx, lambda ast.Expr, sig *types.Signature) {
 	switch expr := lambda.(type) {
-	case *ast.LambdaExpr:
-		if err := compileLambdaExpr(ctx, expr, sig); err != nil {
-			panic(err)
-		}
 	case *ast.LambdaExpr2:
 		if err := compileLambdaExpr2(ctx, expr, sig); err != nil {
 			panic(err)
 		}
+	case *ast.LambdaExpr:
+		if err := compileLambdaExpr(ctx, expr, sig); err != nil {
+			panic(err)
+		}
 	}
 }
 
@@ -1083,9 +1088,11 @@ func compileDomainTextLit(ctx *blockCtx, v *ast.DomainTextLit) {
 	var cb = ctx.cb
 	var imp gogen.PkgRef
 	var name = v.Domain.Name
+	var path string
 	if pi, ok := ctx.findImport(name); ok {
 		imp = pi.PkgRef
-		if pi.Path() == "golang.org/x/net/html" {
+		path = pi.Path()
+		if path == "golang.org/x/net/html" {
 			// html`...` => html.Parse(strings.NewReader(`...`))
 			cb.Val(imp.Ref("Parse")).
 				Val(ctx.pkg.Import("strings").Ref("NewReader")).
@@ -1095,7 +1102,6 @@ func compileDomainTextLit(ctx *blockCtx, v *ast.DomainTextLit) {
 			return
 		}
 	} else {
-		var path string
 		if name == "tpl" {
 			path = tplPkgPath
 		} else {
@@ -1108,9 +1114,60 @@ func compileDomainTextLit(ctx *blockCtx, v *ast.DomainTextLit) {
 		}
 		*/
 	}
-	cb.Val(imp.Ref("New")).
-		Val(&goast.BasicLit{Kind: gotoken.STRING, Value: v.Value}, v).
-		CallWith(1, 0, v)
+
+	n := 1
+	if path == tplPkgPath {
+		pos := ctx.fset.Position(v.ValuePos)
+		filename := relFile(ctx.relBaseDir, pos.Filename)
+		cb.Val(imp.Ref("NewEx")).
+			Val(&goast.BasicLit{Kind: gotoken.STRING, Value: v.Value}, v).
+			Val(filename).Val(pos.Line).Val(pos.Column)
+		n += 3
+		if f, ok := v.Extra.(*tpl.File); ok {
+			decls := f.Decls
+			for _, decl := range decls {
+				if r, ok := decl.(*tpl.Rule); ok {
+					if expr, ok := r.RetProc.(*ast.LambdaExpr2); ok {
+						cb.Val(r.Name.Name)
+						sig := sigRetFunc(ctx.pkg, r.IsList())
+						compileLambdaExpr2(ctx, lambdaRetFunc(expr), sig)
+						n += 2
+					}
+				}
+			}
+		}
+	} else {
+		cb.Val(imp.Ref("New")).
+			Val(&goast.BasicLit{Kind: gotoken.STRING, Value: v.Value}, v)
+	}
+	cb.CallWith(n, 0, v)
+}
+
+func lambdaRetFunc(expr *ast.LambdaExpr2) *ast.LambdaExpr2 {
+	v := *expr
+	v.Lhs = []*ast.Ident{
+		{NamePos: expr.Pos(), Name: "self"},
+	}
+	return &v
+}
+
+func sigRetFunc(pkg *gogen.Package, isList bool) *types.Signature {
+	rets := types.NewTuple(anyParam(pkg))
+	var args *types.Tuple
+	if isList {
+		args = types.NewTuple(anySliceParam(pkg))
+	} else {
+		args = rets
+	}
+	return types.NewSignatureType(nil, nil, nil, args, rets, false)
+}
+
+func anyParam(pkg *gogen.Package) *types.Var {
+	return pkg.NewParam(token.NoPos, "", gogen.TyEmptyInterface)
+}
+
+func anySliceParam(pkg *gogen.Package) *types.Var {
+	return pkg.NewParam(token.NoPos, "", types.NewSlice(gogen.TyEmptyInterface))
 }
 
 const (
