@@ -20,12 +20,12 @@ import (
 	"go/types"
 	"strings"
 
+	"github.com/goplus/gogen"
+	"github.com/goplus/gogen/typeutil"
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/cl"
 	"github.com/goplus/gop/token"
-	"github.com/goplus/gox"
 	"github.com/goplus/mod/modfile"
-	"golang.org/x/tools/go/types/typeutil"
 )
 
 // -----------------------------------------------------------------------------
@@ -37,13 +37,6 @@ type Config struct {
 	// If Fset is nil, Load will use a new fileset, but preserve Fset's value.
 	Fset *token.FileSet
 
-	// C2goBase specifies base of standard c2go packages.
-	// Default is github.com/goplus/.
-	C2goBase string
-
-	// LookupPub lookups the c2go package pubfile (named c2go.a.pub).
-	LookupPub func(pkgPath string) (pubfile string, err error)
-
 	// LookupClass lookups a class by specified file extension.
 	LookupClass func(ext string) (c *Project, ok bool)
 
@@ -53,15 +46,13 @@ type Config struct {
 
 type Package struct {
 	pkg  *types.Package
-	docs gox.ObjectDocs
+	docs gogen.ObjectDocs
 }
 
 // NewPackage creates a Go/Go+ outline package.
 func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (_ Package, err error) {
 	ret, err := cl.NewPackage(pkgPath, pkg, &cl.Config{
 		Fset:           conf.Fset,
-		C2goBase:       conf.C2goBase,
-		LookupPub:      conf.LookupPub,
 		LookupClass:    conf.LookupClass,
 		Importer:       conf.Importer,
 		NoFileLine:     true,
@@ -362,7 +353,7 @@ func (p Var) Doc() string {
 
 type Func struct {
 	*types.Func
-	docs gox.ObjectDocs
+	docs gogen.ObjectDocs
 }
 
 func (p Func) Obj() types.Object {
@@ -523,7 +514,7 @@ func (p Type) CheckNamed(pkg Package) (_ Named, ok bool) {
 
 type Named struct {
 	*types.Named
-	docs gox.ObjectDocs
+	docs gogen.ObjectDocs
 }
 
 func (p Named) Methods() []Func {
