@@ -34,7 +34,7 @@ import (
 )
 
 func checkPathExist(path string, isDir bool) bool {
-	stat, err := os.Stat(path)
+	stat, err := os.Lstat(path) // Note: os.Lstat() will not follow the symbolic link.
 	isExists := !os.IsNotExist(err)
 	if isDir {
 		return isExists && stat.IsDir()
@@ -246,11 +246,17 @@ func getBuildDateTime() string {
 }
 
 func getBuildVer() string {
-	stdout, err := execCommand("git", "describe", "--tags")
+	latestTagCommit, err := execCommand("git", "rev-list", "--tags", "--max-count=1")
 	if err != nil {
 		return ""
 	}
-	return trimRight(stdout)
+
+	stdout, err := execCommand("git", "describe", "--tags", trimRight(latestTagCommit))
+	if err != nil {
+		return ""
+	}
+
+	return fmt.Sprintf("%s devel", trimRight(stdout))
 }
 
 func getGopBuildFlags() string {
@@ -359,6 +365,19 @@ func buildGoplusTools(useGoProxy bool) {
 	println("\nGo+ tools built successfully!")
 }
 
+func goVersion() string {
+	out, err := execCommand("go", "version")
+	if err != nil {
+		log.Fatalln(err)
+	}
+	if i := strings.Index(out, "go1."); i != -1 {
+		if n := strings.IndexAny(out[i+4:], ". "); n != -1 {
+			return out[i+2 : i+4+n]
+		}
+	}
+	return ""
+}
+
 func showHelpPostInstall(installPath string) {
 	println("\nNEXT STEP:")
 	println("\nWe just installed Go+ into the directory: ", installPath)
@@ -542,10 +561,28 @@ func releaseNewVersion(tag string) {
 	println("Released new version:", version)
 }
 
+func runRegtests() {
+	println("\nStart running regtests.")
+
+	cmd := exec.Command(filepath.Join(gopRoot, "bin/"+gopBinFiles[0]), "go", "./...")
+	cmd.Stdout = os.Stdout
+	cmd.Stderr = os.Stderr
+	cmd.Dir = filepath.Join(gopRoot, "demo")
+	err := cmd.Run()
+	if err != nil {
+		code := cmd.ProcessState.ExitCode()
+		if code == 0 {
+			code = 1
+		}
+		os.Exit(code)
+	}
+}
+
 func main() {
 	isInstall := flag.Bool("install", false, "Install Go+")
 	isBuild := flag.Bool("build", false, "Build Go+ tools")
 	isTest := flag.Bool("test", false, "Run testcases")
+	isRegtest := flag.Bool("regtest", false, "Run regtests")
 	isUninstall := flag.Bool("uninstall", false, "Uninstall Go+")
 	isGoProxy := flag.Bool("proxy", false, "Set GOPROXY for people in China")
 	isAutoProxy := flag.Bool("autoproxy", false, "Check to set GOPROXY automatically")
@@ -566,10 +603,11 @@ func main() {
 		},
 		isUninstall: uninstall,
 		isTest:      runTestcases,
+		isRegtest:   runRegtests,
 	}
 
 	// Sort flags, for example: install flag should be checked earlier than test flag.
-	flags := []*bool{isBuild, isInstall, isTest, isUninstall}
+	flags := []*bool{isBuild, isInstall, isTest, isRegtest, isUninstall}
 	hasActionDone := false
 
 	if *tag != "" {
