@@ -57,7 +57,7 @@ type FileSystem = fsx.FileSystem
 
 // Parse parses a single Go+ source file. The filename specifies the Go+ source file.
 // If the file couldn't be read, a nil map and the respective error are returned.
-func Parse(fset *token.FileSet, filename string, src interface{}, mode Mode) (pkgs map[string]*ast.Package, err error) {
+func Parse(fset *token.FileSet, filename string, src any, mode Mode) (pkgs map[string]*ast.Package, err error) {
 	file, err := ParseFile(fset, filename, src, mode)
 	if err != nil {
 		return
@@ -96,7 +96,7 @@ func ParseDirEx(fset *token.FileSet, path string, conf Config) (pkgs map[string]
 }
 
 // ParseEntry calls ParseFSEntry by passing a local filesystem.
-func ParseEntry(fset *token.FileSet, filename string, src interface{}, conf Config) (f *ast.File, err error) {
+func ParseEntry(fset *token.FileSet, filename string, src any, conf Config) (f *ast.File, err error) {
 	return ParseFSEntry(fset, fsx.Local, filename, src, conf)
 }
 
@@ -192,7 +192,7 @@ func ParseFSDir(fset *token.FileSet, fs FileSystem, dir string, conf Config) (pk
 
 // ParseFSEntry parses the source code of a single Go+ source file and returns the corresponding ast.File node.
 // Compared to ParseFSFile, ParseFSEntry detects fileKind by its filename.
-func ParseFSEntry(fset *token.FileSet, fs FileSystem, filename string, src interface{}, conf Config) (f *ast.File, err error) {
+func ParseFSEntry(fset *token.FileSet, fs FileSystem, filename string, src any, conf Config) (f *ast.File, err error) {
 	fname := fs.Base(filename)
 	ext := path.Ext(fname)
 	var isProj, isClass, isNormalGox bool
@@ -322,12 +322,12 @@ func ParseFSEntries(fset *token.FileSet, fs FileSystem, files []string, conf Con
 // -----------------------------------------------------------------------------
 
 // ParseFile parses the source code of a single Go+ source file and returns the corresponding ast.File node.
-func ParseFile(fset *token.FileSet, filename string, src interface{}, mode Mode) (f *ast.File, err error) {
+func ParseFile(fset *token.FileSet, filename string, src any, mode Mode) (f *ast.File, err error) {
 	return ParseFSFile(fset, fsx.Local, filename, src, mode)
 }
 
 // ParseFSFile parses the source code of a single Go+ source file and returns the corresponding ast.File node.
-func ParseFSFile(fset *token.FileSet, fs FileSystem, filename string, src interface{}, mode Mode) (f *ast.File, err error) {
+func ParseFSFile(fset *token.FileSet, fs FileSystem, filename string, src any, mode Mode) (f *ast.File, err error) {
 	code, err := readSourceFS(fs, filename, src)
 	if err != nil {
 		return
@@ -338,7 +338,7 @@ func ParseFSFile(fset *token.FileSet, fs FileSystem, filename string, src interf
 	return parseFile(fset, filename, code, mode)
 }
 
-func readSourceFS(fs FileSystem, filename string, src interface{}) ([]byte, error) {
+func readSourceFS(fs FileSystem, filename string, src any) ([]byte, error) {
 	if src != nil {
 		return iox.ReadSource(src)
 	}
